<?php

namespace MSDev\MSClientAPI;

class RequestResponse {
	
	/** 
	 * The content type of any submitted data 
	 * @var string
	 */
	protected $ct;
	
	/** 
	 * The desired response format for any returned data 
	 * @var string
	 */
	protected $rf;
	
	/** 
	 * Any data submitted with the request
	 * @var string   
	 */
	protected $data;
	
	/**
	 * Dependency container
	 * @var object
	 */
	protected $env;
	
	public function __construct($env) {
		$this->env				= $env;
		
		$headers 				= getallheaders();
		$this->ct				= $this->setContentType($headers);
		$this->rf				= $this->setResponseFormat($headers);
		
		$this->data				= $this->setData();
	}
	
	
	private function setContentType($headers) {
		return $this->extractHeader($headers, 'Content-Type');
	}
	
	private function setResponseFormat($headers) {
		$resp 					= $this->extractHeader($headers, 'Accept');
		if($resp === false) {
			$resp				= 'json';
		}
		
		return $resp;
	}
	
	public function getData() {
		return $this->data;
	}
	
	private function setData() {
		if($this->ct === false) {
			return array();
		}
		
		$input					= file_get_contents('php://input');
		switch($this->ct) {
			case 'json':
				return $this->extractJSON($input);
				break;
				
			case 'xml':
				return $this->extractXML($input);
				break;
		}	
	}
	
	private function extractHeader($headers, $header) {
		if(isset($headers[$header])) {
			switch(strtolower($headers[$header])) {
				case 'application/json':
					return 'json';
					break;
				case 'application/xml':
					return 'xml';
					break;
			}
		}
		
		return false;
	}
	
	private function extractJSON($input) {
		return json_decode($input, true);
	}
	
	private function extractXML($input) {
		$xml					= simplexml_load_string($input);
		return $this->extractJSON(json_encode($xml));
	}
	
	public function respond($type, $data) {
		switch($this->rf) {
			case 'json':
				return $this->responseJSON($type, $data);
				break;
			case 'xml':
				return $this->responseXML($type, $data);
				break;
		}
	}
	
	private function responseJSON($type, $data) {
		header('Content-Type: application/json');
		$resp			= array($type => $data);
		return json_encode($resp);
	}
	
	private function responseXML($type, $data) {
		header('Content-Type: application/xml');
		$xml			= new \SimpleXMLElement("<{$type}></{$type}>");
		$singType		= substr($type, 0, -1);
		$this->recurseXML($xml, $data, $singType);
		return $xml->asXML();
	}
	
	private function recurseXML(&$xml, $data, $type) {
		foreach($data as $k => $v) {
			if(is_array($v)) {
				$child	= $xml->addChild($type);
				$this->recurseXML($child, $v, $k);
			} else {
				$xml->addChild($k, $v);
			}
		}
	}
	
	public function cleanData($data) {
		$permitted		= $this->env->get('PremittedFields');

		$out			= array();
		foreach($data as $k => $v) {
			if(array_key_exists(strtolower($k), $permitted)) {
				$out[$permitted[strtolower($k)]] = $v;
			}
		}
		
		return $out;
	}
}